using System;
using System.Collections.Generic;
using System.Linq;
using Maze.Services;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Maze
{
    public class BGame : Game
    {
        public readonly bool DrawHitboxes = true;

        GraphicsDeviceManager graphics;
        public SpriteBatch SpriteBatch { get; set; }
        public Effect Effect { get; set; }
        public GameTime GameTime { get; set; }

        public EnvironmentService EnvironmentService { get; private set; }
        public CameraService CameraService { get; private set; }
        public PlayerService PlayerService { get; private set; }
        public InputService InputService { get; private set; }
        public InformationService InformationService { get; private set; }
        public DebugService DebugService { get; set; }

        public BGame()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferMultiSampling = true;
            graphics.PreferredBackBufferHeight = 1080;
            graphics.PreferredBackBufferWidth = 1920;
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            Effect = Content.Load<Effect>("Effects/test2");
            EnvironmentService = new EnvironmentService(this);
            CameraService = new CameraService(this);
            InputService = new InputService(this);
            DebugService = new DebugService(this);
            InformationService = new InformationService(this);
            PlayerService = new PlayerService(this, new Vector3(96, 0, 96));

            base.Initialize();
        }

        protected override void LoadContent()
        {
            SpriteBatch = new SpriteBatch(GraphicsDevice);
        }

        protected override void UnloadContent()
        {
        }
        
        protected override void Update(GameTime gameTime)
        {
            
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            ShowFPS(gameTime);
            InformationService.Lines.Clear();
            InputService.Update();
            GameTime = gameTime;
            EnvironmentService.Update(gameTime);
            PlayerService.Update(gameTime);
            CameraService.Update(gameTime);
            DebugService.Update();


            base.Update(gameTime);
        }

        float deltaFPSTime = 0;
        public void ShowFPS(GameTime gameTime)
        {
            // The time since Update was called last
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            float fps = 1 / elapsed;
            deltaFPSTime += elapsed;
            if (deltaFPSTime > 1)
            {

                Window.Title = "I am running at  <" + fps.ToString() + "> FPS";
                deltaFPSTime -= 1;
            }

        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            CameraService.Draw(Effect);

            SpriteBatch.Begin();
            InformationService.Draw();
            SpriteBatch.End();

            DebugService.Draw();

            base.Draw(gameTime);
        }
    }
}
