﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Maze.Models.Visual;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Maze.Helpers
{
    public class ModelLoader
    {
        private BGame _game;
        private Dictionary<string, ModelWrapper> _archive;

        public ModelLoader(BGame game)
        {
            _game = game;
            _archive = new Dictionary<string, ModelWrapper>();
        }

        public ModelWrapper GetModel(string modelname)
        {
            if (_archive.ContainsKey(modelname))
                return _archive[modelname];

            var mw = new ModelWrapper();

            mw.Model = LoadModel("Models/" + modelname + "/" + modelname);
            mw.Diffuse = _game.Content.Load<Texture2D>("Models/" + modelname + "/diffuse");
            mw.Specular = _game.Content.Load<Texture2D>("Models/" + modelname + "/spec");
            mw.Normal = _game.Content.Load<Texture2D>("Models/" + modelname + "/normal");

            _archive.Add(modelname, mw);

            return mw;
        }

        private Model LoadModel(string assetName)
        {
            Model newModel = _game.Content.Load<Model>(assetName); foreach (ModelMesh mesh in newModel.Meshes)
                foreach (ModelMeshPart meshPart in mesh.MeshParts)
                    meshPart.Effect = _game.Effect.Clone();
            return newModel;
        }
    }
}
