﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Maze.Helpers;
using Maze.Models.SectorNamespace;
using Maze.Services;
using Microsoft.Xna.Framework;

namespace Maze.Models.Maps
{
    public class Map
    {
        private BGame _game;
        private Random _rng;
        private Vector3 _scanner;
        private GameTime _gameTime;
        private int _counter;

        public ISector Start { get; set; }
        public ISector LastDetectedSector { get; set; }
        public Dictionary<Vector3, ISector> BuildedSectors { get; set; }
        public Dictionary<Vector3, ISector> CreatedSectors { get; set; }

        #region Dictionaries
        private Dictionary<Directions, int> DirectionX = new Dictionary<Directions, int>
		{
			{ Directions.North, 0 },
			{ Directions.South, 0 },
			{ Directions.East, -1 },
			{ Directions.West, 1 }
		};

        private Dictionary<Directions, int> DirectionY = new Dictionary<Directions, int>
		{
			{ Directions.North, 1 },
			{ Directions.South, -1 },
			{ Directions.East, 0 },
			{ Directions.West, 0 }
		};

        private Dictionary<Directions, Directions> Opposite = new Dictionary<Directions, Directions>
		{
			{ Directions.North, Directions.South },
			{ Directions.South, Directions.North },
			{ Directions.East, Directions.West },
			{ Directions.West, Directions.East }
		};
        #endregion

        public Map(BGame game)
        {
            _counter = 0;
            _game = game;
            _rng = new Random();
            _scanner = new Vector3(0, 0, 10000);

            BuildedSectors = new Dictionary<Vector3, ISector>();
            CreatedSectors = new Dictionary<Vector3, ISector>();

            Start = new Sector(_game, Vector3.Zero);
            Start.Preceder = Start;
            AddSector(Start);

            var timer = new Timer(() => Dig(Start, 100));
            timer.Run("Dig");
            
        }

        public void AddSector(ISector sector)
        {
            CreatedSectors.Add(sector.SectorCoordinate, sector);
        }

        public void RenderSector(ISector sector)
        {
            if (!CreatedSectors.ContainsKey(sector.Key) &&
               !BuildedSectors.ContainsKey(sector.Key))
                throw new Exception("Shit is so fucked up");


            if (!BuildedSectors.ContainsKey(sector.Key))
            {
                sector.BuildModels();
                CreatedSectors.Remove(sector.Key);
                BuildedSectors.Add(sector.Key, sector);
            }

            sector.Draw();
        }

        public bool ContainsKey(Vector3 key)
        {
            return CreatedSectors.ContainsKey(key) ||
                   BuildedSectors.ContainsKey(key);
        }

        public ISector GetSector(Vector3 key)
        {
            if (BuildedSectors.ContainsKey(key))
                return BuildedSectors[key];

            if (CreatedSectors.ContainsKey(key))
                return CreatedSectors[key];

            return null;
        }


        public void Dig(ISector current, int length)
        {
            var candidates = GetCandidates(current.SectorCoordinate);

            if(candidates.Any())
            {
                if (length < 0)
                    return;

                var l = length - 1;
                foreach (var candidate in candidates)
                {
                    current.Directions |= candidate;

                    
                    if (_rng.NextDouble() < 0.1d)
                    {
                        current.Directions |= candidate;
                        var roomSector = new Sector(_game,
                                                    new Vector3(current.SectorCoordinate.X + DirectionX[candidate],
                                                                0,
                                                                current.SectorCoordinate.Z + DirectionY[candidate]));
                        roomSector.Directions |= Opposite[candidate];
                        roomSector.Preceder = current;
                        current.Successors.Add(roomSector);
                        roomSector.IsDeadEnd = true;
                        AddSector(roomSector);

                        //var timer = new Timer(() => CreateRoom(roomSector));
                        //timer.Run("Room");

                        CreateRoom(roomSector);

                        continue;
                    }

                    var newSector = new Sector(_game, new Vector3(current.SectorCoordinate.X + DirectionX[candidate],
                                                                  0,
                                                                  current.SectorCoordinate.Z + DirectionY[candidate]));


                    newSector.Directions |= Opposite[candidate];
                    newSector.Preceder = current;
                    current.Successors.Add(newSector);

                    AddSector(newSector);

                    Dig(newSector, l);
                }
            }
            else
            {
                if (current.SectorCoordinate != Start.SectorCoordinate)
                    Dig(current.Preceder, length);
            }
        }

        public IEnumerable<Directions> GetCandidates(Vector3 s)
        {
            var directions = new List<Directions> { Directions.North, Directions.South, Directions.East, Directions.West };
            directions.Shuffle(_rng);

            foreach (Directions direction in directions)
            {
                var nextX = s.X + DirectionX[direction];
                var nextY = s.Z + DirectionY[direction];
                var candidate = new Vector3(nextX, 0, nextY);

                if (!this.ContainsKey(candidate))
                    yield return direction;
            }
        }

        public void Update(GameTime gameTime)
        {
            _counter = 0;
            _gameTime = gameTime;
            //var t = 360*gameTime.ElapsedGameTime.TotalMilliseconds/1000;

            //_scanner = Vector3.Transform(_scanner, Matrix.CreateRotationY(MathHelper.ToRadians((float) -t)));

            //_game.DebugService.AddVector("scanner", _game.PlayerService.Player.Position ,_scanner);

            var playersector = new Vector3(
                (int)Math.Ceiling((_game.PlayerService.Player.Position.X + 0) / 192) - 1,
                0,
                (int)Math.Ceiling((_game.PlayerService.Player.Position.Z + 0) / 192) - 1
                );

            if (this.ContainsKey(playersector))
                LastDetectedSector = this.GetSector(playersector);
        }

        public void TestAround(float angle, Vector3 target)
        {
            _counter++;
            if (angle >= 360)
                return;

            var player = _game.PlayerService.Player.Position;

            var v = target - player;
            v.Y = 0;
            v.Normalize();
            v = v*1000;

            _game.DebugService.AddVector("scanner" + _counter, player, v - player);

            Ray r = new Ray(new Vector3(_game.PlayerService.Player.Position.X, 30, _game.PlayerService.Player.Position.Z), v);

            float? flo = null;
            foreach (var n in CreatedSectors.Values) // gotta fix this
            {
                flo = n.ScanAndDraw(r);
                if (flo != null)
                {
                    //_game.DebugService.AddPoint("scanner" + _counter, (Vector3)(r.Position + r.Direction * flo.Value));
                    break;
                }
            }

            if(flo != null)
            {
                var ins = r.Position + r.Direction*flo.Value;
                
                //r.Direction.Normalize();
                
                var turn = Vector3.Transform(r.Direction, Matrix.CreateRotationY(MathHelper.ToRadians(90)));

                ins += turn;
                ins.Y = 0;
                var vec = ins - player;
                var angl = FindAngleBetweenTwoVectors(new Vector2(v.X, v.Z), new Vector2(vec.X, vec.Z));
                TestAround(
                    angle + angl
                    , ins + turn);
            }

        }

        public float FindAngleBetweenTwoVectors(Vector2 v1, Vector2 v2)
        {
            float angle;
            // turn vectors into unit vectors   
            v1.Normalize();
            v2.Normalize();

            angle = (float) Math.Acos(Vector2.Dot(v1, v2));
            // if no noticable rotation is available return zero rotation  
            // this way we avoid Cross product artifacts   
            if (Math.Abs(angle) < 0.0001)
                return 0;
            angle *= signal(v1, v2);

            return angle;
        }

        int signal(Vector2 v1, Vector2 v2)
        {
            return (v1.Y * v2.X - v2.Y * v1.X) > 0 ? 1 : -1;
        }  

        public void Draw()
        {
            var playersector = new Vector3(
                (int)Math.Ceiling((_game.PlayerService.Player.Position.X + 0) / 192) - 1,
                0,
                (int)Math.Ceiling((_game.PlayerService.Player.Position.Z + 0) / 192) - 1
                );

            _game.InformationService.Lines.Add("Player Sector: " + playersector.X + " " + playersector.Y + " " +
                                               playersector.Z);

            LastDetectedSector = this.GetSector(playersector);
            DrawSectors(LastDetectedSector, 5);
            //var player = _game.PlayerService.Player.Position;
            //var head = new Vector3(player.X, 30, player.Z);

            ////TestAround(0, _scanner);

            //for (int i = 0; i < 360; i = i + 10)
            //{
            //    var v = Vector3.Transform(_scanner, Matrix.CreateRotationY(MathHelper.ToRadians((float)-i)));
            //    Ray r = new Ray(head, v);

            //    //_game.DebugService.AddVector("scanner" + i, head, v);

            //    foreach (var n in CreatedSectors.Values.Take(100))
            //    {
            //        float? f = n.ScanAndDraw(r);
            //        if (f != null)
            //        {
            //            _game.DebugService.AddPoint("scanner" + i, (Vector3)(r.Position + r.Direction * f.Value));
            //            break;
            //        }
            //    }
            //}
        }

        private void DrawSectors(ISector s, int depth, ISector previous = null)
        {
            if (s == null)
                return;

            RenderSector(s);

            if (depth > 0)
            {
                var nei = previous == null
                              ? s.Neighbours
                              : s.Neighbours.Where(x => x.SectorCoordinate != previous.SectorCoordinate);          
                
                foreach (var n in nei)
                {
                    DrawSectors(n, depth - 1, s);
                }
            }
        }





        public void CarvePassagesFrom(ISector current, int length)
        {
            var directions = new List<Directions> { Directions.North, Directions.South, Directions.East, Directions.West };
            directions.Shuffle(_rng);

            foreach (Directions direction in directions)
            {
                var nextX = current.SectorCoordinate.X + DirectionX[direction];
                var nextY = current.SectorCoordinate.Z + DirectionY[direction];
                var testpos = new Vector3(nextX, 0, nextY);
                //if (nextX > 9 || nextX < 0 || nextY < 0 || nextY > 9)
                //    continue;

                if (CreatedSectors.ContainsKey(testpos) || BuildedSectors.ContainsKey(testpos))
                    continue;

                if (CreatedSectors.Count + BuildedSectors.Count > length)
                    break;

                //if (rng.NextDouble() < 0.3d)
                //{
                //    current.Directions |= direction;
                //    var roomSector = new Sector(_game, testpos);
                //    roomSector.Directions |= Opposite[direction];
                //    roomSector.Neighbours.Add(current);
                //    current.Neighbours.Add(roomSector);
                //    roomSector.IsDeadEnd = true;
                //    Sectors.Add(roomSector.SectorCoordinate, roomSector);

                //    //CreateRoom(roomSector, 1);


                //    continue;
                //}

                current.Directions |= direction;

                var newSector = new Sector(_game, testpos);
                newSector.Directions |= Opposite[direction];
                newSector.Preceder = current;
                current.Successors.Add(newSector);

                AddSector(newSector);

                CarvePassagesFrom(newSector, length);
            }
        }

        public void SolveBack(ISector sector, int distance, ISector final)
        {
            sector.DistanceToEnd = distance;

            foreach (var n in sector.Successors.Where(x => x.DistanceToEnd == int.MaxValue))
            {
                if (n.SectorCoordinate != final.SectorCoordinate)
                    SolveBack(n, sector.DistanceToEnd + 1, final);
                else
                    n.DistanceToEnd = sector.DistanceToEnd + 1;
            }
        }

        public void CreateRoom(ISector current)
        {
            //var timer = new Timer(() => DigRoom(current, _rng.Next(4), Guid.NewGuid()));
            //            timer.Run("Dig Room");
            DigRoom(current, _rng.Next(4), Guid.NewGuid());

            //var timer2 = new Timer(() => MergeRoom(current));
            //            timer2.Run("Merge Room");
            MergeRoom(current);
            
        }

        public void DigRoom(ISector current, int depth, Guid id)
        {
            if (depth > 0)
            {
                var d = depth - 1;
                var directions = new List<Directions> { Directions.North, Directions.South, Directions.East, Directions.West };
                directions.Shuffle(_rng);


                foreach (var direction in directions)
                {
                    var nextX = current.SectorCoordinate.X + DirectionX[direction];
                    var nextY = current.SectorCoordinate.Z + DirectionY[direction];
                    var testpos = new Vector3(nextX, 0, nextY);

                    if (this.ContainsKey(testpos))
                    {
                        continue;
                    }

                    current.Directions |= direction;
                    var newSector = new Sector(_game, testpos);
                    newSector.RoomID = id;
                    newSector.IsDeadEnd = true;
                    newSector.Directions |= Opposite[direction];
                    newSector.Preceder = current;
                    current.Successors.Add(newSector);

                    AddSector(newSector);
                    DigRoom(newSector, d, id);
                }
            }
        }

        public void MergeRoom(ISector head)
        {
            foreach (ISector s in head.Successors)
            {
                var directions = new List<Directions> { Directions.North, Directions.South, Directions.East, Directions.West };
                directions.Shuffle(_rng);

                foreach (Directions direction in directions)
                {
                    var v = s.GetNeighbour(direction);
                    if (CreatedSectors.ContainsKey(v) && CreatedSectors[v].RoomID == s.RoomID)
                    {
                        s.Directions |= direction;
                        CreatedSectors[v].Directions |= Opposite[direction];
                    }
                }

                MergeRoom(s);
            }
        }
    }
}
