﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Maze.Models.SectorNamespace;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Maze.Models.TileNamespace
{
    public class Tile : Renderable
    {
        private TimeSpan _lastDrawn;
        private float _scale;
        VertexPositionColor[] points;
        short[] index;
        BasicEffect lineShader;

        public Sector Parent { get; set; }

        public Tile(BGame game, string modelname, Vector3 position, float rotation, Sector parent, float scale = 0.790f)
            : base(game, modelname, position, rotation)
        {
            _lastDrawn = new TimeSpan(0, 0, 0);
            Parent = parent;
            _scale = scale;
            //this.Position = new Vector3(Position.X + 32, Position.Y, Position.Z - 32);
        }

        protected void BuildBox(BoundingBox box, Color lineColor)
        {
            points = new VertexPositionColor[8];

            Vector3[] corners = box.GetCorners();

            points[0] = new VertexPositionColor(corners[1], lineColor); // Front Top Right
            points[1] = new VertexPositionColor(corners[0], lineColor); // Front Top Left
            points[2] = new VertexPositionColor(corners[2], lineColor); // Front Bottom Right
            points[3] = new VertexPositionColor(corners[3], lineColor); // Front Bottom Left
            points[4] = new VertexPositionColor(corners[5], lineColor); // Back Top Right
            points[5] = new VertexPositionColor(corners[4], lineColor); // Back Top Left
            points[6] = new VertexPositionColor(corners[6], lineColor); // Back Bottom Right
            points[7] = new VertexPositionColor(corners[7], lineColor); // Bakc Bottom Left

            index = new short[] {
	            0, 1, 0, 2, 1, 3, 2, 3,
	            4, 5, 4, 6, 5, 7, 6, 7,
	            0, 4, 1, 5, 2, 6, 3, 7
                };
        }

        protected void DrawLines(int primativeCount)
        {
            if (points == null)
                BuildBox(this.Hitbox, Color.AliceBlue);

            if (lineShader == null)
                lineShader = new BasicEffect(_game.GraphicsDevice);

            lineShader.World = _game.CameraService.WorldMatrix;
            lineShader.View = _game.CameraService.ViewMatrix;
            lineShader.Projection = _game.CameraService.ProjectionMatrix;
            lineShader.VertexColorEnabled = true;

            lineShader.CurrentTechnique.Passes[0].Apply();
            _game.GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.LineList, points, 0, points.Length, index, 0, primativeCount);
        }

        public override void Draw()
        {
            if (_lastDrawn == _game.GameTime.TotalGameTime)
                return;


            Matrix worldMatrix = Matrix.CreateScale(_scale) *
                Rotation *
                Matrix.CreateTranslation(Position);

            Matrix[] transforms = new Matrix[Model.Model.Bones.Count];
            Model.Model.CopyAbsoluteBoneTransformsTo(transforms);

            var translation = Matrix.CreateTranslation(new Vector3(0, 0, 0));

            //Vector3 lightpos = Parent.Coordinate - Position;
            //Vector3 lightpos = Parent.Coordinate;
            Vector3 lightpos = _game.PlayerService.Player.Position;
            lightpos.Y = 100;

            foreach (ModelMesh mesh in Model.Model.Meshes)
            {
                foreach (var effect in mesh.Effects)
                {
                    var w = transforms[mesh.ParentBone.Index] * worldMatrix * translation;

                    effect.Parameters["Diffuse"].SetValue(Model.Diffuse);
                    effect.Parameters["Specular"].SetValue(Model.Specular);
                    effect.Parameters["Normal"].SetValue(Model.Normal);
                    effect.Parameters["Ambient"].SetValue(new Vector3(0.050f, 0.045f, 0.025f));

                    effect.Parameters["light1Dir"].SetValue(Position);
                    effect.Parameters["light1Pos"].SetValue(lightpos);
                    effect.Parameters["light1Color"].SetValue(Color.AliceBlue.ToVector4());
                    effect.Parameters["light1Attenuation"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 0.1f));
                    effect.Parameters["light1Hotspot"].SetValue(43f);
                    effect.Parameters["light1Falloff"].SetValue(45f);

                    effect.Parameters["wvp"].SetValue(w * _game.CameraService.ViewMatrix * _game.CameraService.ProjectionMatrix);
                    effect.Parameters["worldI"].SetValue(Matrix.Invert(w));
                    effect.Parameters["worldIT"].SetValue(Matrix.Transpose(Matrix.Invert(w)));

                    effect.Parameters["viewInv"].SetValue(Matrix.Invert(_game.CameraService.ViewMatrix));

                    effect.Parameters["world"].SetValue(w);

                    foreach (var pass in effect.CurrentTechnique.Passes)
                    {
                        pass.Apply();
                        mesh.Draw();
                    }
                }
            }

            _lastDrawn = _game.GameTime.TotalGameTime;

            if(_game.DrawHitboxes)
                DrawLines(12);
        }
    }
}
