﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Maze.Services
{
    public class CameraService : ServiceBase
    {
        public Vector3 CameraPosition { get; set; }
        public Vector3 RelativePosition { get; set; }
        public Matrix ViewMatrix { get; set; }
        public Matrix ProjectionMatrix { get; set; }
        public Matrix WorldMatrix { get; set; }

        public Vector3 LookPosition { get; set; }

        private MouseState newMouseState, oldMouseState;
        private int totalCamPitch;
        private int totalCamYaw;
        private Matrix cameraRotation;
        private float Zoom { get; set; }

        private const int CamDistance = 500;

        public CameraService(BGame game) : base(game)
        {
            Zoom = 500;
            CameraPosition = new Vector3(0, 0, CamDistance);
            LookPosition = new Vector3(0, 0, 0);
            ViewMatrix = Matrix.CreateLookAt(CameraPosition, LookPosition, new Vector3(0, 1, 0));
            ProjectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, _game.GraphicsDevice.Viewport.AspectRatio, 0.2f, 15000.0f);
            WorldMatrix = Matrix.Identity;

            //_game.Effect.Parameters["light1Dir"].SetValue(new Vector3(0, 0, 0));
            //_game.Effect.Parameters["light1Pos"].SetValue(new Vector3(0, 100, 0));
            //_game.Effect.Parameters["light1Color"].SetValue(Color.White.ToVector4());
            //_game.Effect.Parameters["light1Attenuation"].SetValue(Color.White.ToVector4());
            //_game.Effect.Parameters["light1Hotspot"].SetValue(43f);
            //_game.Effect.Parameters["light1Falloff"].SetValue(45f);
        }

        public void Update(GameTime gameTime)
        {
            LookPosition = _game.PlayerService.Player.Position;

            HandleMouseCam();

            CameraPosition = new Vector3(CameraPosition.X + _game.PlayerService.Player.Position.X,
                                         CameraPosition.Y + _game.PlayerService.Player.Position.Y,
                                         CameraPosition.Z + _game.PlayerService.Player.Position.Z);
        }

        private void HandleMouseCam()
        {
            oldMouseState = newMouseState;
            newMouseState = Mouse.GetState();

            var a = newMouseState.ScrollWheelValue - oldMouseState.ScrollWheelValue;
            Zoom += a;
            Vector3 cameraOffset = new Vector3(0, 0, Zoom);

            if (newMouseState.LeftButton == ButtonState.Pressed)
            {
                _game.IsMouseVisible = false;

                totalCamPitch += (oldMouseState.Y - newMouseState.Y) / 5;
                totalCamYaw += (oldMouseState.X - newMouseState.X) / 5;

                if (totalCamPitch >= 89)
                {
                    totalCamPitch = 89;
                }
                if (totalCamPitch <= -89)
                {
                    totalCamPitch = -89;
                }
            }
            else
            {
                _game.IsMouseVisible = true;
            }

            cameraRotation = Matrix.CreateRotationX(MathHelper.ToRadians(totalCamPitch)) *
                Matrix.CreateRotationY(MathHelper.ToRadians(totalCamYaw));

            Vector3 cameraRotatedPosition = Vector3.Transform(cameraOffset, cameraRotation);
            CameraPosition = cameraRotatedPosition + LookPosition;
            RelativePosition = cameraRotatedPosition + LookPosition;

            ViewMatrix = Matrix.CreateLookAt(CameraPosition, LookPosition, new Vector3(0, 1, 0));
            
        }

        public void Draw(Effect effect)
        {
            _game.EnvironmentService.Draw();
            _game.PlayerService.Draw();
        }
    }
}
