﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Maze.Models;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Maze.Services
{
    public class DebugService : ServiceBase
    {
        //public List<VertexPositionColorTexture> Vectors { get; set; }
        private Dictionary<string, Renderable> Vectors { get; set; }
        private Dictionary<string, Renderable> Points { get; set; }

        private BasicEffect _effect;
        private Texture2D _texture;

        public DebugService(BGame game)
            : base(game)
        {
            Vectors = new Dictionary<string, Renderable>();
            Points = new Dictionary<string, Renderable>();
        }

        public void AddPoint(string name, Vector3 p)
        {
            if(Points.ContainsKey(name))
            {
                Points.Remove(name);
            }

            Points.Add(name, new Renderable(_game,"zDebugBall",p,0.0f,0.2f));
        }

        public void AddVector(string name, Vector3 p, Vector3 v)
        {
            if (Vectors.ContainsKey(name))
            {
                Vectors.Remove(name);
            }

            Matrix rotation = GetRotationMatrix(new Vector3(0, 1, 0), v);

            Vectors.Add(name, new Renderable(_game, "zDebugArrow", p, rotation));
        }

        public void Update()
        {
            
        }

        public void Draw()
        {
            foreach (var v in Vectors.Values)
            {
                v.Draw();
            }

            foreach (var r in Points.Values)
            {
                r.Draw();
            }
           
        }

        public Matrix GetRotationMatrix(Vector3 source, Vector3 target)
        {
            float dot = Vector3.Dot(source, target);
            if (!float.IsNaN(dot))
            {
                float angle = (float)Math.Acos(dot);
                if (!float.IsNaN(angle))
                {
                    Vector3 cross = Vector3.Cross(source, target);
                    cross.Normalize();
                    Matrix rotation = Matrix.CreateFromAxisAngle(cross, angle);
                    return rotation;
                }
            }
            return Matrix.Identity;
        }
    }
}
