﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;

namespace Maze.Services
{
    public class InputService : ServiceBase
    {
        private KeyboardState _currentKeyboardState;
        private KeyboardState _previousKeyboardState;
        private Plane _floor;
        public Vector3 MouseOnFloor { get; set; }

        public InputService(BGame game) : base(game)
        {
            _floor = new Plane(new Vector3(0, 1, 0), 0);
        }

        public void Update()
        {
            _previousKeyboardState = _currentKeyboardState;
            _currentKeyboardState = Keyboard.GetState();
            MouseState mouseState = Mouse.GetState();

            if (_currentKeyboardState.IsKeyDown(Keys.A) ||
                _currentKeyboardState.IsKeyDown(Keys.S) ||
                _currentKeyboardState.IsKeyDown(Keys.D) ||
                _currentKeyboardState.IsKeyDown(Keys.W))
            {
                _game.PlayerService.MovePlayer(_currentKeyboardState, _previousKeyboardState);
            }

            
            int mouseX = mouseState.X;
            int mouseY = mouseState.Y;

            Vector3 nearsource = new Vector3((float)mouseX, (float)mouseY, 0f);
            Vector3 farsource = new Vector3((float)mouseX, (float)mouseY, 1f);

            Matrix world = Matrix.CreateTranslation(0, 0, 0);

            Vector3 nearPoint = _game.GraphicsDevice.Viewport.Unproject(nearsource, _game.CameraService.ProjectionMatrix, _game.CameraService.ViewMatrix, _game.CameraService.WorldMatrix);

            Vector3 farPoint = _game.GraphicsDevice.Viewport.Unproject(farsource, _game.CameraService.ProjectionMatrix, _game.CameraService.ViewMatrix, _game.CameraService.WorldMatrix);

            Vector3 direction = farPoint - nearPoint;
            direction.Normalize();

            var e = _game.CameraService.RelativePosition.Y / direction.Y;

            MouseOnFloor = _game.PlayerService.Player.Position - (direction * e);

            Ray r = new Ray(_game.CameraService.RelativePosition, direction);

            float? f = r.Intersects(_floor);

            Vector3 final = Vector3.Zero;

            if (f.HasValue)
            {
                final = r.Position + r.Direction * f.Value;
            }

            _game.DebugService.AddPoint("target", final);
            _game.DebugService.AddVector("target", _game.PlayerService.Player.Position, final - _game.PlayerService.Player.Position);

        }
    }
}
